@@ -2,13 +2,15 @@
 '''Command line arguments parser
 '''
 
-import sys, traceback, getopt, types, textwrap, inspect, os, copy, keyword
+import sys, traceback, getopt, types, textwrap, inspect, os, keyword, codecs
 from itertools import imap
 from functools import wraps
+from collections import namedtuple, Callable
+from contextlib import contextmanager
 
 
 __all__ = ['Dispatcher', 'command', 'dispatch']
-__version__ = '3.4'
+__version__ = '3.7'
 __author__ = 'Alexander Solovyov'
 __email__ = 'alexander@solovyov.net'
 
@@ -17,27 +19,32 @@ try:
     import locale
     ENCODING = locale.getpreferredencoding()
     if (not ENCODING or ENCODING == 'mac-roman' or 'ascii' in ENCODING.lower()
-        or 'ansi' in ENCODING.lower()):
+        or 'ansi' in ENCODING.lower() or ENCODING.lower() == 'cp1252'):
         ENCODING = 'UTF-8'
 except locale.Error:
     ENCODING = 'UTF-8'
 
+_writer = codecs.getwriter(ENCODING)
+
 
 def write(text, out=None):
-    '''Write output to a given stream (stdout by default)'''
+    '''Write output to a given stream (stdout by default).'''
     out = out or sys.stdout
-    if sys.version_info < (3, 0) and isinstance(text, unicode):
+    if hasattr(out, 'buffer'):
+        out = _writer(out.buffer)
+    elif sys.version_info < (3, 0) and isinstance(text, unicode):
         text = text.encode(ENCODING)
     out.write(text)
 
 
 def err(text):
-    '''Write output to stderr'''
+    '''Write output to stderr.'''
     write(text, out=sys.stderr)
+    sys.stderr.flush()
 
 
 class Dispatcher(object):
-    '''Central object for command dispatching system
+    '''Central object for command dispatching system.
 
     - ``cmdtable``: dict of commands. Will be populated with functions,
       decorated with ``Dispatcher.command``.
@@ -48,14 +55,14 @@ class Dispatcher(object):
 
     def __init__(self, cmdtable=None, globaloptions=None, middleware=None):
         self._cmdtable = cmdtable or {}
-        self._globaloptions = globaloptions or []
+        self._globaloptions = [Option(o) for o in (globaloptions or [])]
         self.middleware = middleware
 
     @property
     def globaloptions(self):
         opts = self._globaloptions[:]
-        if not next((True for o in opts if o[1] == 'help'), None):
-            opts.append(('h', 'help', False, 'display help'))
+        if not any(o.name == 'help' for o in opts):
+            opts.append(Option(('h', 'help', False, 'display help')))
         return opts
 
     @property
@@ -117,7 +124,7 @@ class Dispatcher(object):
         '''
         def wrapper(func):
             try:
-                options_ = list(options or guess_options(func))
+                options_ = [Option(o) for o in (options or guess_options(func))]
             except TypeError:
                 options_ = []
 
@@ -139,29 +146,30 @@ class Dispatcher(object):
 
             def command(argv=None):
                 for o in self.globaloptions:
-                    if not next((x for x in options_ if
-                                 x[1] == o[1] or (x[0] and x[0] == o[0])),
-                                None):
-                        options_.append(o)
+                    # Don't include global option if long name matches
+                    if any((x.name == o.name for x in options_)):
+                        continue
+                    # Don't use global option short name if already used
+                    if any((x.short and x.short == o.short
+                            for x in options_)):
+                        o = o._replace(short='')
+                    options_.append(o)
 
                 if argv is None:
                     argv = sys.argv[1:]
-                try:
-                    args, opts = process(argv, options_)
-                except Exception, e:
-                    if exchandle(e, func.help):
-                        return -1
-                    raise
-
-                if opts.pop('help', False):
-                    return func.help()
 
                 try:
-                    return call_cmd(scriptname, func, options_)(*args, **opts)
-                except Exception, e:
-                    if exchandle(e, func.help):
-                        return -1
-                    raise
+                    with exchandle(func.help):
+                        args, opts = process(argv, options_)
+
+                    if opts.pop('help', False):
+                        return func.help()
+
+                    with exchandle(func.help):
+                        return call_cmd(scriptname, func, options_)(*args, **opts)
+
+                except ErrorHandled:
+                    return -1
 
             func.usage = usage_
             func.help = help_func
@@ -177,26 +185,8 @@ class Dispatcher(object):
 
         return wrapper
 
-    def _dispatch(self, args):
-        cmd, func, args, options = cmdparse(args, self.cmdtable,
-                                            self.globaloptions)
-        try:
-            args, kwargs = process(args, options)
-        except getopt.GetoptError, e:
-            # FIXME: this is ugly, we set command name here to retrieve it later
-            # in exchandle().
-            e.command = cmd
-            raise
-
-        if kwargs.pop('help', False):
-            return 'help', self.cmdtable['help'][0], [cmd], {}, options
-        if not cmd:
-            return 'help', self.cmdtable['help'][0], ['shortlist'], {}, options
-
-        return cmd, func, args, kwargs, options
-
     def dispatch(self, args=None):
-        '''Dispatch command line arguments using subcommands
+        '''Dispatch command line arguments using subcommands.
 
         - ``args``: list of arguments, default: ``sys.argv[1:]``
         '''
@@ -206,19 +196,24 @@ class Dispatcher(object):
         autocomplete(self.cmdtable, args, self.middleware)
 
         try:
-            name, func, args, kwargs, options = self._dispatch(args)
-        except Exception, e:
-            if exchandle(e, help_func):
-                return -1
-            raise
+            with exchandle(help_func):
+                cmd, func, args, options = cmdparse(args, self.cmdtable,
+                                                    self.globaloptions)
 
-        try:
-            mw = name != '_completion' and self.middleware or None
-            return call_cmd(name, func, options, mw)(*args, **kwargs)
-        except Exception, e:
-            if exchandle(e, help_func):
-                return -1
-            raise
+            with exchandle(help_func, cmd):
+                args, opts = process(args, options)
+
+            if not cmd:
+                cmd, func, args, opts = ('help', help_func, ['shortlist'], {})
+            if opts.pop('help', False):
+                cmd, func, args, opts = ('help', help_func, [cmd], {})
+
+            mw = cmd != '_completion' and self.middleware or None
+            with exchandle(help_func, cmd):
+                return call_cmd(cmd, func, options, mw)(*args, **opts)
+
+        except ErrorHandled:
+            return -1
 
 
 _dispatcher = None
@@ -242,7 +237,7 @@ def dispatch(args=None, cmdtable=None, globaloptions=None, middleware=None):
         if cmdtable:
             _dispatcher._cmdtable = cmdtable
         if globaloptions:
-            _dispatcher._globaloptions = globaloptions
+            _dispatcher._globaloptions = [Option(o) for o in globaloptions]
         if middleware:
             _dispatcher.middleware = middleware
     return _dispatcher.dispatch(args)
@@ -254,10 +249,10 @@ dispatch.__doc__ = Dispatcher.dispatch.__doc__
 # --------
 
 def help_(cmdtable, globalopts):
-    '''Help generator for a command table
+    '''Help generator for a command table.
     '''
     def help_inner(name=None, **opts):
-        '''Show help for a given help topic or a help overview
+        '''Show help for a given help topic or a help overview.
 
         With no arguments, print a list of commands with short help messages.
 
@@ -302,7 +297,7 @@ def help_(cmdtable, globalopts):
 
 
 def help_cmd(func, usage, options, aliases):
-    '''show help for given command
+    '''Show help for given command.
 
     - ``func``: function to generate help for (``func.__doc__`` is taken)
     - ``usage``: usage string
@@ -335,6 +330,7 @@ def help_cmd(func, usage, options, aliases):
      -d --daemonize  daemonize process
         --pid-file   name of file to write process ID to
     '''
+    options = [Option(o) for o in options]  # only for doctest
     write(usage + '\n')
     if aliases:
         write('\naliases: ' + ', '.join(aliases) + '\n')
@@ -345,18 +341,16 @@ def help_cmd(func, usage, options, aliases):
 
 
 def help_options(options):
-    '''Generator for help on options
+    '''Generator for help on options.
     '''
     yield 'options:\n\n'
     output = []
     for o in options:
-        short, name, default, desc = o[:4]
-        if hasattr(default, '__call__'):
-            default = default(None)
+        default = o.default_value()
         default = default and ' (default: %s)' % default or ''
-        output.append(('%2s%s' % (short and '-%s' % short,
-                                  name and ' --%s' % name),
-                       '%s%s' % (desc, default)))
+        output.append(('%2s%s' % (o.short and '-%s' % o.short,
+                                  o.name and ' --%s' % o.name),
+                       '%s%s' % (o.helpmsg, default)))
 
     opts_len = max([len(first) for first, second in output if second] or [0])
     for first, second in output:
@@ -373,7 +367,138 @@ def help_options(options):
 # Options process
 # --------
 
-def process(args, options, preparse=False):
+# Factory for creating _Option instances. Intended to be the entry point to
+# the *Option classes here.
+def Option(opt):
+    '''Create Option instance from tuple of option data.'''
+    if isinstance(opt, BaseOption):
+        return opt
+
+    # Extract and validate contents of tuple
+    short, name, default, helpmsg = opt[:4]
+    completer = opt[4] if len(opt) > 4 else None
+    if short and len(short) != 1:
+        raise OpsterError(
+            'Short option should be only a single character: %s' % short)
+    if not name:
+        raise OpsterError(
+            'Long name should be defined for every option')
+    pyname = name_to_python(name)
+
+    args = pyname, name, short, default, helpmsg, completer
+
+    # Find matching _Option subclass and return instance
+    # nb. the order of testing matters
+    for Type in (BoolOption, IntOption, FloatOption, ListOption,
+                 DictOption, FuncOption, LiteralOption):
+        if Type.matches(default):
+            return Type(*args)
+    raise OpsterError('Cannot figure out type for option %s' % name)
+
+
+# Superclass for all option classes
+class BaseOption(namedtuple('Option', (
+            'pyname', 'name', 'short', 'default', 'helpmsg', 'completer'))):
+    has_parameter = True
+    type = None
+
+    def __repr__(self):
+        return (super(BaseOption, self).__repr__()
+                .replace('Option', self.__class__.__name__, 1))
+
+    @classmethod
+    def matches(cls, default):
+        '''Returns True if this is appropriate Option for the default value.'''
+        return isinstance(default, cls.type)
+
+    def default_state(self):
+        '''Generate initial state value from provided default value.'''
+        return self.default
+
+    def update_state(self, state, new):
+        '''Update state after encountering an option on the command line.'''
+        return new
+
+    def convert(self, final):
+        '''Generate the resulting python value from the final state.'''
+        return self.type(final)
+
+    def default_value(self):
+        '''Shortcut to obtain the default value when option arg not provided.'''
+        return self.convert(self.default_state())
+
+
+class LiteralOption(BaseOption):
+    '''Literal option type (including string options, no processing).'''
+    type = object
+
+    def convert(self, final):
+        return final
+
+
+class BoolOption(BaseOption):
+    '''Boolean option type.'''
+    has_parameter = False
+    type = (bool, types.NoneType)
+
+    def convert(self, final):
+        return bool(final)
+
+    def update_state(self, state, new):
+        return not self.default
+
+
+class IntOption(BaseOption):
+    '''Integer number option type.'''
+    type = int
+
+
+class FloatOption(BaseOption):
+    '''Floating point number option type.'''
+    type = float
+
+
+class ListOption(BaseOption):
+    '''List option type.'''
+    type = list
+
+    def default_state(self):
+        return list(self.default)
+
+    def update_state(self, state, new):
+        state.append(new)
+        return state
+
+
+class DictOption(BaseOption):
+    '''Dict option type.'''
+    type = dict
+
+    def default_state(self):
+        return dict(self.default)
+
+    def update_state(self, state, new):
+        try:
+            k, v = new.split('=')
+        except ValueError:
+            msg = "wrong definition: %r (should be in format KEY=VALUE)"
+            raise getopt.GetoptError(msg % new)
+        state[k] = v
+        return state
+
+
+class FuncOption(BaseOption):
+    '''Function option type.'''
+    type = Callable
+
+    def default_state(self):
+        return None
+
+    def convert(self, final):
+        return self.default(final)
+
+
+def process(args, options):
     '''
     >>> opts = [('l', 'listen', 'localhost',
     ...          'ip to listen on'),
@@ -387,124 +512,92 @@ def process(args, options, preparse=False):
     (['all'], {'pid_file': 'test', 'daemonize': False, 'port': 8000, 'listen': '0.0.0.0'})
 
     '''
-    argmap, defmap, state = {}, {}, {}
-    shortlist, namelist, funlist = '', [], []
+    options = [Option(o) for o in options]  # only for doctest
 
+    # Parse arguments and options
+    args, opts = getopts(args, options)
+
+    # Default values
+    state = dict((o.pyname, o.default_state()) for o in options)
+
+    # Update for each option on the command line
+    for o, val in opts:
+        state[o.pyname] = o.update_state(state[o.pyname], val)
+
+    # Convert to required type
     for o in options:
-        # might have the fifth completer element
-        short, name, default, comment = o[:4]
-        if short and len(short) != 1:
-            raise OpsterError(
-                'Short option should be only a single character: %s' % short)
-        if not name:
-            raise OpsterError(
-                'Long name should be defined for every option')
-        # change name to match Python styling
-        pyname = name_to_python(name)
-        argmap['-' + short] = argmap['--' + name] = pyname
-        defmap[pyname] = default
-
-        # copy defaults to state
-        if isinstance(default, (list, dict)):
-            state[pyname] = copy.copy(default)
-        elif isinstance(default, types.FunctionType):
-            funlist.append(pyname)
-            state[pyname] = None
-        else:
-            state[pyname] = default
-
-        # getopt wants indication that it takes a parameter
-        if not (default is None or default is True or default is False):
-            if short:
-                short += ':'
-            if name:
-                name += '='
-        if short:
-            shortlist += short
-        if name:
-            namelist.append(name)
-
-    try:
-        opts, args = getopt.gnu_getopt(args, shortlist, namelist)
-    except getopt.GetoptError, e:
-        if preparse:
-            prefix = '-' if len(e.opt) == 1 else '--'
-            args = args[:]
-            args.insert(args.index(prefix + e.opt), '--')
-            opts, args = getopt.gnu_getopt(args, shortlist, namelist)
-            return args, None
-        raise
-
-    # transfer result to state
-    for opt, val in opts:
-        name = argmap[opt]
-        t = type(defmap[name])
-        if t is types.FunctionType:
-            del funlist[funlist.index(name)]
-            state[name] = defmap[name](val)
-        elif t is list:
-            state[name].append(val)
-        elif t is dict:
-            try:
-                k, v = val.split('=')
-            except ValueError:
-                raise getopt.GetoptError(
-                    "wrong definition: %r (should be in format KEY=VALUE)"
-                    % val)
-            state[name][k] = v
-        elif t in (types.NoneType, types.BooleanType):
-            state[name] = not defmap[name]
-        elif t in (int, float):
-            try:
-                state[name] = t(val)
-            except ValueError:
-                raise getopt.GetoptError(
-                    'invalid option value %r for option %r'
-                    % (val, name))
-        else:
-            state[name] = t(val)
-
-    for name in funlist:
-        state[name] = defmap[name](None)
+        try:
+            state[o.pyname] = o.convert(state[o.pyname])
+        except ValueError:
+            raise getopt.GetoptError('invalid option value %r for option %r'
+                % (state[o.pyname], o.name))
 
     return args, state
 
 
+def getopts(args, options, preparse=False):
+    '''Parse args and options from raw args.
+
+    If preparse is True, option processing stops at first non-option.
+    '''
+    argmap = {}
+    shortlist, namelist = '', []
+    for o in options:
+        argmap['-' + o.short] = argmap['--' + o.name] = o
+
+        # getopt wants indication that it takes a parameter
+        short, name = o.short, o.name
+        if o.has_parameter:
+            if short:
+                short += ':'
+            name += '='
+        if short:
+            shortlist += short
+        namelist.append(name)
+
+    # gnu_getopt will stop at first non-option argument
+    if preparse:
+        shortlist = '+' + shortlist
+
+    # getopt.gnu_getopt allows options after the first non-option
+    opts, args = getopt.gnu_getopt(args, shortlist, namelist)
+
+    # map the option argument names back to their Option instances
+    opts = [(argmap[opt], val) for opt, val in opts]
+
+    return args, opts
+
 # --------
 # Subcommand system
 # --------
 
 def cmdparse(args, cmdtable, globalopts):
-    '''Parse arguments list to find a command, options and arguments
+    '''Parse arguments list to find a command, options and arguments.
     '''
     # pre-parse arguments here using global options to find command name,
     # which is first non-option entry
-    cmd = next((arg for arg in process(args, globalopts, preparse=True)[0]
-                if not arg.startswith('-')), None)
+    args_new, opts = getopts(args, globalopts, preparse=True)
 
-    if cmd:
-        args.pop(args.index(cmd))
-
-        aliases, info = findcmd(cmd, cmdtable)
+    args = list(args)
+    if args_new:
+        cmdarg = args_new[0]
+        args.remove(cmdarg)
+        aliases, info = findcmd(cmdarg, cmdtable)
         cmd = aliases[0]
-        possibleopts = list(info[1])
+        possibleopts = list(info[1]) + globalopts
+        return cmd, info[0] or None, args, possibleopts
     else:
-        possibleopts = []
-
-    possibleopts.extend(globalopts)
-    return cmd, cmd and info[0] or None, args, possibleopts
+        return None, None, args, globalopts
 
 
 def aliases_(cmdtable_key):
-    '''Get aliases from a command table key'''
+    '''Get aliases from a command table key.'''
     return cmdtable_key.lstrip("^~").split("|")
 
 
 def findpossible(cmd, table):
-    """
-    Return cmd -> (aliases, command table entry)
-    for each matching command.
-    """
+    '''Return cmd -> (aliases, command table entry) for each matching command.
+    '''
     choice = {}
     for e in table.keys():
         aliases = aliases_(e)
@@ -558,9 +651,7 @@ def guess_options(func):
     for name, option in zip(args[-len(defaults):], defaults):
         if not isinstance(option, tuple):
             continue
-        sname, default, hlp = option[:3]
-        completer = option[3] if len(option) > 3 else None
-        yield (sname, name_from_python(name), default, hlp, completer)
+        yield (option[0], name_from_python(name)) + option[1:]
 
 
 def guess_usage(func, options):
@@ -570,7 +661,7 @@ def guess_usage(func, options):
     if options:
         usage.append('[OPTIONS]')
     arginfo = inspect.getargspec(func)
-    optnames = [x[1] for x in options]
+    optnames = [o.name for o in options]
     nonoptional = len(arginfo.args) - len(arginfo.defaults or ())
 
     for i, arg in enumerate(arginfo.args):
@@ -583,32 +674,35 @@ def guess_usage(func, options):
     return ' '.join(usage)
 
 
-def exchandle(e, help_func):
-    '''Handle internal exceptions and print human-readable information on them
+@contextmanager
+def exchandle(help_func, cmd=None):
+    '''Context manager to turn internal exceptions into printed help messages.
 
-    Returns False is exception is not suitable
+    Handles internal opster exceptions by printing help and raising
+    ErrorHandled. Other exceptions are propagated.
     '''
-    if isinstance(e, UnknownCommand):
+    try:
+        yield  # execute the block in the 'with' statement
+        return
+    except UnknownCommand as e:
         err("unknown command: '%s'\n" % e)
-    elif isinstance(e, AmbiguousCommand):
+    except AmbiguousCommand as e:
         err("command '%s' is ambiguous:\n    %s\n" %
             (e.args[0], ' '.join(e.args[1])))
-    elif isinstance(e, ParseError):
+    except ParseError as e:
         err('%s: %s\n\n' % (e.args[0], e.args[1].strip()))
-        help_func(e.args[0])
-    elif isinstance(e, getopt.GetoptError):
+        help_func(cmd)
+    except getopt.GetoptError as e:
         err('error: %s\n\n' % e)
-        # we may get command name here, if we're in multicommand context
-        help_func(getattr(e, 'command', None))
-    elif isinstance(e, OpsterError):
+        help_func(cmd)
+    except OpsterError as e:
         err('%s\n' % e)
-    else:
-        return False
-    return True
+    # abort if a handled exception was raised
+    raise ErrorHandled()
 
 
 def call_cmd(name, func, opts, middleware=None):
-    '''Wrapper for command call, catching situation with insufficient arguments
+    '''Wrapper for command call, catching situation with insufficient arguments.
     '''
     # depth is necessary when there is a middleware in setup
     arginfo = inspect.getargspec(func)
@@ -627,11 +721,10 @@ def call_cmd(name, func, opts, middleware=None):
         start = None
         if arginfo.varargs and len(args) > (len(arginfo.args) - len(kwargs)):
             for o in opts:
-                optname = o[1].replace('-', '_')
-                if optname in arginfo.args:
+                if o.pyname in arginfo.args:
                     if start is None:
-                        start = arginfo.args.index(optname)
-                    prepend.append(optname)
+                        start = arginfo.args.index(o.pyname)
+                    prepend.append(o.pyname)
             if start is not None:  # do we have to prepend anything
                 args = (args[:start] +
                         tuple(kwargs.pop(x) for x in prepend) +
@@ -647,7 +740,7 @@ def call_cmd(name, func, opts, middleware=None):
 
 
 def call_cmd_regular(func, opts):
-    '''Wrapper for command for handling function calls from Python
+    '''Wrapper for command for handling function calls from Python.
     '''
     def inner(*args, **kwargs):
         arginfo = inspect.getargspec(func)
@@ -656,8 +749,7 @@ def call_cmd_regular(func, opts):
                             ' than applicable')
 
         # short name, long name, default, help, (maybe) completer
-        funckwargs = dict((o[1].replace('-', '_'), o[2])
-                          for o in opts)
+        funckwargs = dict((o.pyname, o.default) for o in opts)
         if 'help' not in (arginfo.defaults or ()) and not arginfo.keywords:
             funckwargs.pop('help', None)
         funckwargs.update(kwargs)
@@ -666,24 +758,24 @@ def call_cmd_regular(func, opts):
 
 
 def replace_name(usage, name):
-    '''Replace name placeholder with a command name'''
+    '''Replace name placeholder with a command name.'''
     if '%name' in usage:
         return usage.replace('%name', name, 1)
     return name + ' ' + usage
 
 
 def sysname():
-    '''Returns name of executing file'''
+    '''Returns name of executing file.'''
     name = sys.argv[0]
-    if name.startswith('/'):
-        return name.rsplit('/', 1)[1]
+    if os.path.isabs(name):
+        return name.rsplit(os.sep, 1)[1]
     elif name.startswith('./'):
         return name[2:]
     return name
 
 
 def pretty_doc_string(item):
-    "Doc string with adjusted indentation level of the 2nd line and beyond."
+    '''Doc string with adjusted indentation level of the 2nd line and beyond.'''
     raw_doc = item.__doc__ or '(no help text available)'
     lines = raw_doc.strip().splitlines()
     if len(lines) <= 1:
@@ -711,10 +803,10 @@ def name_to_python(name):
 
 # Borrowed from PIP
 def autocomplete(cmdtable, args, middleware):
-    """Command and option completion.
+    '''Command and option completion.
 
     Enable by sourcing one of the completion shell scripts (bash or zsh).
-    """
+    '''
 
     # Don't complete if user hasn't sourced bash_completion file.
     if 'OPSTER_AUTO_COMPLETE' not in os.environ:
@@ -742,12 +834,12 @@ def autocomplete(cmdtable, args, middleware):
         aliases, (cmd, opts, usage) = findcmd(cwords[0], cmdtable)
 
         for o in opts:
-            short, long, default, help = o[:4]
-            completer = o[4] if len(o) > 4 else None
-            short, long = '-%s' % short, '--%s' % long
-            options += [short, long]
+            short = '-%s' % o.short
+            name = '--%s' % o.name
+            options += [short, name]
 
-            if cwords[idx] in (short, long) and completer:
+            completer = o.completer
+            if cwords[idx] in (short, name) and completer:
                 if middleware:
                     completer = middleware(completer)
                 args = completer(current)
@@ -760,7 +852,7 @@ def autocomplete(cmdtable, args, middleware):
 
 COMPLETIONS = {
     'bash':
-        """
+        '''
 # opster bash completion start
 _opster_completion()
 {
@@ -770,9 +862,9 @@ _opster_completion()
 }
 complete -o default -F _opster_completion %s
 # opster bash completion end
-""",
+''',
     'zsh':
-            """
+            '''
 # opster zsh completion start
 function _opster_completion {
   local words cword
@@ -784,7 +876,7 @@ function _opster_completion {
 }
 compctl -K _opster_completion %s
 # opster zsh completion end
-"""
+'''
     }
 
 
@@ -793,7 +885,7 @@ def completion(type=('t', 'bash', 'Completion type (bash or zsh)'),
                # kwargs will catch every global option, which we get
                # anyway, because middleware is skipped
                **kwargs):
-    """Outputs completion script for bash or zsh."""
+    '''Outputs completion script for bash or zsh.'''
 
     prog_name = os.path.split(sys.argv[0])[1]
     print COMPLETIONS[type].strip() % prog_name
@@ -819,6 +911,18 @@ class ParseError(OpsterError):
     'Raised on error in command line parsing'
 
 
+class QuitError(OpsterError):
+    'Raised to exit script with a message to the user'
+
+
+class ErrorHandled(OpsterError):
+    'Raised to signal that opster is aborting the command'
+
+
+# API to expose QuitError for opster users
+command.Error = QuitError
+
+
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
